import { ReactNode } from 'react';
import { History, MemoryHistoryOptions } from 'history';
import type { Route, RouterInstance } from '../types';
import defaultResolve from '../resolve-view';
type Props = {
    children: ReactNode;
    routes: Route[] | Route;
    baseUrl?: string;
    resolveView?: typeof defaultResolve;
    errorHandler?: (error: Error) => ReactNode;
};
/**
 * Base Router Component.
 * @group Components
 */
export declare function BaseRouter(props: Props & {
    history: History;
}): JSX.Element;
/**
 * History mode Router Component.
 * @group Components
 */
declare function RouterComponent(props: Props): JSX.Element;
export { RouterComponent as Router };
/**
 * Hash mode Router Component.
 * @group Components
 */
export declare function HashRouter(props: Props): JSX.Element;
/**
 * Memory mode Router Component.
 * @group Components
 */
export declare function MemoryRouter({ initialEntries, initialIndex, ...props }: Props & MemoryHistoryOptions): JSX.Element;
/**
 * Get Router instance.
 * @group Hooks
 * @returns Router Instance
 */
export declare function useRouter(): RouterInstance<Route, ReactNode>;
